/* $Id: rotmatvec3.h,v 1.3 2006/01/18 12:02:28 cvsroot Exp $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_ROTMATVEC3_INCLUDED
#define GENLIB2_ROTMATVEC3_INCLUDED

#include "rotmatrix3.h"
#include "vector3.h"

using namespace std;

namespace GenLib2
{

/** matrix-vector multiplication.
 *  c = A * b. Apply the rotation A on the point/vector b.
 */
template<class T>
void mult(Vector3<T>& c, const RotMatrix3<T>& A, const Vector3<T>& b)
{
	c.x = A(0,0) * b.x + A(0,1) * b.y + A(0,2) * b.z;
	c.y = A(1,0) * b.x + A(1,1) * b.y + A(1,2) * b.z;
	c.z = A(2,0) * b.x + A(2,1) * b.y + A(2,2) * b.z;
}


/** inverse matrix-vector multiplication.
 *  c = trn(A) * b. Apply the inverse rotation A on the point/vector b.
 */
template<class T>
void mult_inv(Vector3<T>& c, const RotMatrix3<T>& A, const Vector3<T>& b)
{
	c.x = A(0,0) * b.x + A(1,0) * b.y + A(2,0) * b.z;
	c.y = A(0,1) * b.x + A(1,1) * b.y + A(2,1) * b.z;
	c.z = A(0,2) * b.x + A(1,2) * b.y + A(2,2) * b.z;
}


/** matrix-vector multiplication.
 *  Apply the rotation A on the point/vector b.
 */
template<class T>
Vector3<T> operator*(const RotMatrix3<T>& A, const Vector3<T>& b)
{
	Vector3<T> c;

	//c.x = A(0,0) * b.x + A(0,1) * b.y + A(0,2) * b.z;
	//c.y = A(1,0) * b.x + A(1,1) * b.y + A(1,2) * b.z;
	//c.z = A(2,0) * b.x + A(2,1) * b.y + A(2,2) * b.z;
	mult(c, A, b);

	return c;
}

} // namespace GenLib2

#endif /* GENLIB2_ROTMATVEC3_INCLUDED */ 
